@@ -44,6 +44,8 @@
 #define INCLUDED_MBOXNAME_H
 
 #include "auth.h"
+#include "strarray.h"
+#include "util.h"
 
 #define MAX_NAMESPACE_PREFIX 40
 
@@ -51,10 +53,19 @@
 #define DOTCHAR '^'
 
 /* list of our namespaces */
-enum { NAMESPACE_INBOX = 0, 
-       NAMESPACE_USER = 1, 
+enum { NAMESPACE_INBOX = 0,
+       NAMESPACE_USER = 1,
        NAMESPACE_SHARED = 2 };
 
+/* categorise mailboxes */
+enum { MBNAME_INBOX = 1,
+       MBNAME_INBOXSUB = 2,
+       MBNAME_ALTINBOX = 3,
+       MBNAME_ALTPREFIX = 4,
+       MBNAME_OWNER = 5,
+       MBNAME_OTHERUSER = 6,
+       MBNAME_SHARED = 7 };
+
 /* structure holding server namespace info */
 struct namespace {
     char hier_sep;
@@ -62,43 +73,57 @@ struct namespace {
     int isadmin; /* current user is an admin */
     char prefix[3][MAX_NAMESPACE_PREFIX+1];
     int accessible[3];
-    /* Convert the external mailbox 'name' to an internal name. */
-    int (*mboxname_tointernal)(struct namespace *namespace, const char *name,
-			       const char *userid, char *result);
-    /* Convert the internal mailbox 'name' to an external name. */
-    int (*mboxname_toexternal)(struct namespace *namespace, const char *name,
-			       const char *userid, char *result);
-    int (*mboxlist_findall)(struct namespace *namespace,
-			    const char *pattern, int isadmin, const char *userid, 
-			    struct auth_state *auth_state, int (*proc)(),
-			    void *rock);
-    int (*mboxlist_findsub)(struct namespace *namespace,
-			    const char *pattern, int isadmin, const char *userid, 
-			    struct auth_state *auth_state, int (*proc)(),
-			    void *rock, int force);
 };
 
 #define NAMESPACE_INITIALIZER { '.', 0, 0, \
-				{ "INBOX.", "user.", "" }, \
-				{ 0, 0, 0, }, \
-				NULL, NULL, NULL, NULL }
+                                { "INBOX.", "user.", "" }, \
+                                { 0, 0, 0, } }
 
 struct mboxlock {
     char *name;
     int lock_fd;
-    int locktype;
+    int locktype;       /* LOCK_NONE or LOCK_SHARED or LOCK_EXCLUSIVE */
 };
 
-struct mboxname_parts {
-    const char *domain;
-    const char *userid;	    /* userid WITHOUT the domain */
-    const char *box;
-    int is_deleted;
-    char *freeme;
-};
+struct mbname_parts;
+
+typedef struct mbname_parts mbname_t;
+
+const char *mbname_userid(const mbname_t *mbname);
+const char *mbname_intname(const mbname_t *mbname);
+const char *mbname_extname(const mbname_t *mbname, const struct namespace *ns, const char *userid);
+int mbname_category(const mbname_t *mbname, const struct namespace *ns, const char *userid);
+const char *mbname_category_prefix(int category, const struct namespace *ns);
+const char *mbname_domain(const mbname_t *mbname);
+const char *mbname_localpart(const mbname_t *mbname);
+const strarray_t *mbname_boxes(const mbname_t *mbname);
+time_t mbname_isdeleted(const mbname_t *mbname);
+const char *mbname_recipient(const mbname_t *mbname, const struct namespace *ns);
+
+mbname_t *mbname_from_userid(const char *userid);
+mbname_t *mbname_from_localdom(const char *localpart, const char *domain);
+mbname_t *mbname_from_intname(const char *intname);
+mbname_t *mbname_from_extname(const char *extname, const struct namespace *ns, const char *userid);
+mbname_t *mbname_from_extsub(const char *extsub, const struct namespace *ns, const char *userid);
+mbname_t *mbname_from_recipient(const char *recip, const struct namespace *ns);
+mbname_t *mbname_dup(const mbname_t *mbname);
+
+void mbname_downcaseuser(mbname_t *mbname);
+void mbname_set_localpart(mbname_t *mbname, const char *localpart);
+void mbname_set_domain(mbname_t *mbname, const char *domain);
+void mbname_set_isdeleted(mbname_t *mbname, time_t del);
+void mbname_set_boxes(mbname_t *mbname, const strarray_t *boxes);
+void mbname_push_boxes(mbname_t *mbname, const char *item);
+char *mbname_pop_boxes(mbname_t *mbname); /* free it yourself punk */
+void mbname_truncate_boxes(mbname_t *mbname, size_t len);
+void mbname_free(mbname_t **mbnamep);
+
+char *mboxname_from_external(const char *extname, const struct namespace *ns, const char *userid);
+char *mboxname_to_external(const char *intname, const struct namespace *ns, const char *userid);
+
 
 int mboxname_lock(const char *mboxname, struct mboxlock **mboxlockptr,
-		  int locktype);
+                  int locktype);
 void mboxname_release(struct mboxlock **mboxlockptr);
 
 /* Create namespace based on config options. */
@@ -106,32 +131,14 @@ int mboxname_init_namespace(struct namespace *namespace, int isadmin);
 
 struct namespace *mboxname_get_adminnamespace();
 
-/*
- * Translate separator charactors in a mailboxname from its external
- * representation to its internal representation '.'.
- * If using the unixhierarchysep '/', all '.'s get translated to DOTCHAR.
- * length is the length of the string to translate (0 = strlen(name)).
- */
-char *mboxname_hiersep_tointernal(struct namespace *namespace, char *name,
-				  int length);
-
-/*
- * Translate separator charactors in a mailboxname from its internal
- * representation '.' to its external representation.
- * If using the unixhierarchysep '/', all DOTCHAR get translated to '.'.
- * length is the length of the string to translate (0 = strlen(name)).
- */
-char *mboxname_hiersep_toexternal(struct namespace *namespace, char *name,
-                                  int length);
-
 /* Return nonzero if 'userid' owns the (internal) mailbox 'name'. */
 int mboxname_userownsmailbox(const char *userid, const char *name);
 
 /*
  * If (internal) mailbox 'name' is a user's mailbox (optionally INBOX),
- * returns a pointer to the userid, otherwise returns NULL.
+ * returns 1, otherwise returns 0.
  */
-char *mboxname_isusermailbox(const char *name, int isinbox);
+int mboxname_isusermailbox(const char *name, int isinbox);
 
 /*
  * If (internal) mailbox 'name' is in the DELETED namespace.
@@ -141,26 +148,6 @@ char *mboxname_isusermailbox(const char *name, int isinbox);
  */
 int mboxname_isdeletedmailbox(const char *name, time_t *timestampp);
 
-/*
- * Split an (internal) inboxname into it's constituent parts.
- * also: userid
- */
-int mboxname_to_parts(const char *mboxname, struct mboxname_parts *parts);
-int mboxname_userid_to_parts(const char *userid, struct mboxname_parts *parts);
-
-/*
- * Create an (internal) mboxname from parts
- */
-
-int mboxname_parts_to_internal(struct mboxname_parts *parts, char *target);
-
-/*
- * Cleanup up a mboxname_parts structure.
- */
-void mboxname_init_parts(struct mboxname_parts *parts);
-void mboxname_free_parts(struct mboxname_parts *parts);
-
-
 /*
  * If (internal) mailbox 'name' is a CALENDAR mailbox
  * returns boolean
@@ -173,22 +160,50 @@ int mboxname_iscalendarmailbox(const char *name, int mbtype);
  */
 int mboxname_isaddressbookmailbox(const char *name, int mbtype);
 
+/*
+ * If (internal) mailbox 'name' is a DAVDRIVE mailbox
+ * returns boolean
+ */
+int mboxname_isdavdrivemailbox(const char *name, int mbtype);
+
+/*
+ * If (internal) mailbox 'name' is a DAVNOTIFICATIONS mailbox
+ * returns boolean
+ */
+int mboxname_isdavnotificationsmailbox(const char *name, int mbtype);
+
+/* If (internal) mailbox is a user's top-level Notes mailbox,
+ * returns boolean
+ */
+int mboxname_isnotesmailbox(const char *name, int mbtype);
+
+/* If (internal) mailbox is a user's mail outbox,
+ * returns boolean
+ */
+int mboxname_isoutbox(const char *name);
+
 /* check if one mboxname is a parent or same as the other */
 int mboxname_is_prefix(const char *longstr, const char *shortstr);
 
+void mboxname_hash(char *buf, size_t buf_len,
+                   const char *root,
+                   const char *name) ;
+
 /*
  * Translate (internal) inboxname into corresponding userid,
  * and vice-versa.
  */
-const char *mboxname_to_userid(const char *mboxname);
 /* returns a malloc'd mailbox */
+char *mboxname_to_userid(const char *mboxname);
 char *mboxname_user_mbox(const char *userid, const char *subfolder);
+char *mboxname_user_mbox_external(const char *userid, const char *extsubfolder);
+char *mboxname_abook(const char *userid, const char *collection);
+char *mboxname_cal(const char *userid, const char *collection);
 
 /*
  * Check whether two mboxnames have the same userid.
  */
-int mboxname_parts_same_userid(struct mboxname_parts *a,
-			       struct mboxname_parts *b);
+int mbname_same_userid(const mbname_t *a, const mbname_t *b);
 int mboxname_same_userid(const char *mboxname1, const char *mboxname2);
 
 
@@ -196,19 +211,23 @@ int mboxname_same_userid(const char *mboxname1, const char *mboxname2);
  * Access files (or directories by leaving last parameter
  * zero) for a particular mailbox on partition.
  */
-void mboxname_hash(char *buf, size_t buf_len,
-		   const char *root,
-		   const char *name);
+char *mboxname_datapath(const char *partition,
+                        const char *mboxname,
+                        const char *uniqueid,
+                        unsigned long uid);
 
-char *mboxname_datapath(const char *partition, 
-			const char *mboxname,
-			unsigned long uid);
+char *mboxname_archivepath(const char *partition,
+                           const char *mboxname,
+                           const char *uniqueid,
+                           unsigned long uid);
 
 char *mboxname_metapath(const char *partition,
-			const char *mboxname,
-			int metafile, int isnew);
+                        const char *mboxname,
+                        const char *uniqueid,
+                        int metafile, int isnew);
 
 char *mboxname_lockpath(const char *mboxname);
+char *mboxname_lockpath_suffix(const char *mboxname, const char *suffix);
 
 /*
  * Return nonzero if (internal) mailbox 'name' consists of legal characters.
@@ -226,6 +245,31 @@ void mboxname_todeleted(const char *name, char *result, int withtime);
  */
 int mboxname_make_parent(char *namebuf);
 
-char *mboxname_conf_getpath(struct mboxname_parts *parts,
-			    const char *suffix);
+char *mboxname_conf_getpath(const mbname_t *mbname,
+                            const char *suffix);
+
+/* ======================== COUNTERS ==================== */
+
+struct mboxname_counters {
+    uint32_t generation;
+    uint32_t version;
+    modseq_t highestmodseq;
+    modseq_t mailmodseq;
+    modseq_t caldavmodseq;
+    modseq_t carddavmodseq;
+    modseq_t notesmodseq;
+    modseq_t mailfoldersmodseq;
+    modseq_t caldavfoldersmodseq;
+    modseq_t carddavfoldersmodseq;
+    modseq_t notesfoldersmodseq;
+    uint32_t uidvalidity;
+};
+
+int mboxname_read_counters(const char *mboxname, struct mboxname_counters *vals);
+modseq_t mboxname_nextmodseq(const char *mboxname, modseq_t last, int mbtype, int dofolder);
+modseq_t mboxname_setmodseq(const char *mboxname, modseq_t val, int mbtype, int dofolder);
+uint32_t mboxname_readuidvalidity(const char *mboxname);
+uint32_t mboxname_nextuidvalidity(const char *mboxname, uint32_t last);
+uint32_t mboxname_setuidvalidity(const char *mboxname, uint32_t val);
+
 #endif
